@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Gentoo Authors
+# Copyright 1999-2023 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 import argparse
@@ -21,6 +21,7 @@ portage.proxy.lazyimport.lazyimport(
 from portage import os
 from portage.sync import _SUBMODULE_PATH_MAP
 
+from typing import Optional
 
 options = [
     "--alphabetical",
@@ -52,6 +53,7 @@ options = [
     "--tree",
     "--unordered-display",
     "--update",
+    "--update-if-installed",
 ]
 
 shortmapping = {
@@ -81,7 +83,7 @@ shortmapping = {
 
 COWSAY_MOO = r"""
 
-  Larry loves Gentoo (%s)
+  Larry loves Gentoo ({})
 
  _______________________
 < Have you mooed today? >
@@ -97,7 +99,7 @@ COWSAY_MOO = r"""
 
 def multiple_actions(action1, action2):
     sys.stderr.write("\n!!! Multiple actions requested... Please choose one only.\n")
-    sys.stderr.write("!!! '%s' or '%s'\n\n" % (action1, action2))
+    sys.stderr.write(f"!!! '{action1}' or '{action2}'\n\n")
     sys.exit(1)
 
 
@@ -162,6 +164,7 @@ def insert_optional_args(args):
         "--jobs": valid_integers,
         "--keep-going": y_or_n,
         "--load-average": valid_floats,
+        "--onlydeps-with-ideps": y_or_n,
         "--onlydeps-with-rdeps": y_or_n,
         "--package-moves": y_or_n,
         "--quiet": y_or_n,
@@ -472,6 +475,7 @@ def parse_opts(tmpcmdline, silent=False):
             "choices": y_or_n,
         },
         "--exclude": {
+            "shortopt": "-X",
             "help": "A space separated list of package names or slot atoms. "
             + "Emerge won't  install any ebuild or binary package that "
             + "matches any of the given package atoms.",
@@ -572,8 +576,12 @@ def parse_opts(tmpcmdline, silent=False):
             + "Emerge will ignore matching binary packages. ",
             "action": "append",
         },
+        "--onlydeps-with-ideps": {
+            "help": "modify interpretation of dependencies to include IDEPEND",
+            "choices": true_y_or_n,
+        },
         "--onlydeps-with-rdeps": {
-            "help": "modify interpretation of depedencies",
+            "help": "modify interpretation of dependencies",
             "choices": true_y_or_n,
         },
         "--rebuild-exclude": {
@@ -670,7 +678,7 @@ def parse_opts(tmpcmdline, silent=False):
             "action": "store",
         },
         "--root-deps": {
-            "help": "modify interpretation of depedencies",
+            "help": "modify interpretation of dependencies",
             "choices": ("True", "rdeps"),
         },
         "--search-index": {
@@ -744,7 +752,7 @@ def parse_opts(tmpcmdline, silent=False):
 
     for action_opt in actions:
         parser.add_argument(
-            "--" + action_opt,
+            f"--{action_opt}",
             action="store_true",
             dest=action_opt.replace("-", "_"),
             default=False,
@@ -758,7 +766,7 @@ def parse_opts(tmpcmdline, silent=False):
         )
     for shortopt, longopt in shortmapping.items():
         parser.add_argument(
-            "-" + shortopt,
+            f"-{shortopt}",
             action="store_true",
             dest=longopt.lstrip("--").replace("-", "_"),
             default=False,
@@ -831,9 +839,9 @@ def parse_opts(tmpcmdline, silent=False):
     if myoptions.buildpkg_exclude:
         bad_atoms = _find_bad_atoms(myoptions.buildpkg_exclude, less_strict=True)
         if bad_atoms and not silent:
+            invalid_atoms = ",".join(bad_atoms)
             parser.error(
-                "Invalid Atom(s) in --buildpkg-exclude parameter: '%s'\n"
-                % (",".join(bad_atoms),)
+                f"Invalid Atom(s) in --buildpkg-exclude parameter: '{invalid_atoms}'\n"
             )
 
     if myoptions.changed_deps is not None:
@@ -875,52 +883,23 @@ def parse_opts(tmpcmdline, silent=False):
     if myoptions.depclean_lib_check in true_y:
         myoptions.depclean_lib_check = True
 
-    if myoptions.exclude:
-        bad_atoms = _find_bad_atoms(myoptions.exclude)
-        if bad_atoms and not silent:
-            parser.error(
-                "Invalid Atom(s) in --exclude parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
-            )
+    candidate_bad_options = (
+        (myoptions.exclude, "exclude"),
+        (myoptions.reinstall_atoms, "reinstall-atoms"),
+        (myoptions.rebuild_exclude, "rebuild-exclude"),
+        (myoptions.rebuild_ignore, "rebuild-ignore"),
+        (myoptions.usepkg_exclude, "usepkg-exclude"),
+        (myoptions.useoldpkg_atoms, "useoldpkg-atoms"),
+    )
+    bad_options = (
+        (_find_bad_atoms(atoms), flag) for atoms, flag in candidate_bad_options if atoms
+    )
 
-    if myoptions.reinstall_atoms:
-        bad_atoms = _find_bad_atoms(myoptions.reinstall_atoms)
+    for bad_atoms, flag in bad_options:
         if bad_atoms and not silent:
+            invalid_atoms = ",".join(bad_atoms)
             parser.error(
-                "Invalid Atom(s) in --reinstall-atoms parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
-            )
-
-    if myoptions.rebuild_exclude:
-        bad_atoms = _find_bad_atoms(myoptions.rebuild_exclude)
-        if bad_atoms and not silent:
-            parser.error(
-                "Invalid Atom(s) in --rebuild-exclude parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
-            )
-
-    if myoptions.rebuild_ignore:
-        bad_atoms = _find_bad_atoms(myoptions.rebuild_ignore)
-        if bad_atoms and not silent:
-            parser.error(
-                "Invalid Atom(s) in --rebuild-ignore parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
-            )
-
-    if myoptions.usepkg_exclude:
-        bad_atoms = _find_bad_atoms(myoptions.usepkg_exclude)
-        if bad_atoms and not silent:
-            parser.error(
-                "Invalid Atom(s) in --usepkg-exclude parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
-            )
-
-    if myoptions.useoldpkg_atoms:
-        bad_atoms = _find_bad_atoms(myoptions.useoldpkg_atoms)
-        if bad_atoms and not silent:
-            parser.error(
-                "Invalid Atom(s) in --useoldpkg-atoms parameter: '%s' (only package names and slot atoms (with wildcards) allowed)\n"
-                % (",".join(bad_atoms),)
+                f"Invalid Atom(s) in --{flag} parameter: '{invalid_atoms}' (only package names and slot atoms (with wildcards) allowed)\n"
             )
 
     if myoptions.fail_clean in true_y:
@@ -1003,7 +982,6 @@ def parse_opts(tmpcmdline, silent=False):
         myoptions.selective = True
 
     if myoptions.backtrack is not None:
-
         try:
             backtrack = int(myoptions.backtrack)
         except (OverflowError, ValueError):
@@ -1013,7 +991,7 @@ def parse_opts(tmpcmdline, silent=False):
             backtrack = None
             if not silent:
                 parser.error(
-                    "Invalid --backtrack parameter: '%s'\n" % (myoptions.backtrack,)
+                    f"Invalid --backtrack parameter: '{myoptions.backtrack}'\n"
                 )
 
         myoptions.backtrack = backtrack
@@ -1031,7 +1009,7 @@ def parse_opts(tmpcmdline, silent=False):
         if deep is not True and deep < 0:
             deep = None
             if not silent:
-                parser.error("Invalid --deep parameter: '%s'\n" % (myoptions.deep,))
+                parser.error(f"Invalid --deep parameter: '{myoptions.deep}'\n")
 
         myoptions.deep = deep
 
@@ -1048,7 +1026,7 @@ def parse_opts(tmpcmdline, silent=False):
         if jobs is not True and jobs < 1:
             jobs = None
             if not silent:
-                parser.error("Invalid --jobs parameter: '%s'\n" % (myoptions.jobs,))
+                parser.error(f"Invalid --jobs parameter: '{myoptions.jobs}'\n")
 
         myoptions.jobs = jobs
 
@@ -1065,8 +1043,7 @@ def parse_opts(tmpcmdline, silent=False):
             load_average = None
             if not silent:
                 parser.error(
-                    "Invalid --load-average parameter: '%s'\n"
-                    % (myoptions.load_average,)
+                    f"Invalid --load-average parameter: '{myoptions.load_average}'\n"
                 )
 
         myoptions.load_average = load_average
@@ -1081,8 +1058,7 @@ def parse_opts(tmpcmdline, silent=False):
             rebuilt_binaries_timestamp = 0
             if not silent:
                 parser.error(
-                    "Invalid --rebuilt-binaries-timestamp parameter: '%s'\n"
-                    % (myoptions.rebuilt_binaries_timestamp,)
+                    f"Invalid --rebuilt-binaries-timestamp parameter: '{myoptions.rebuilt_binaries_timestamp}'\n"
                 )
 
         myoptions.rebuilt_binaries_timestamp = rebuilt_binaries_timestamp
@@ -1092,14 +1068,12 @@ def parse_opts(tmpcmdline, silent=False):
             search_similarity = float(myoptions.search_similarity)
         except ValueError:
             parser.error(
-                "Invalid --search-similarity parameter "
-                "(not a number): '{}'\n".format(myoptions.search_similarity)
+                f"Invalid --search-similarity parameter (not a number): '{myoptions.search_similarity}'\n"
             )
 
         if search_similarity < 0 or search_similarity > 100:
             parser.error(
-                "Invalid --search-similarity parameter "
-                "(not between 0 and 100): '{}'\n".format(myoptions.search_similarity)
+                f"Invalid --search-similarity parameter (not between 0 and 100): '{myoptions.search_similarity}'\n"
             )
 
         myoptions.search_similarity = search_similarity
@@ -1180,7 +1154,7 @@ def profile_check(trees, myaction):
             "--help, --info, --search, --sync, and --version."
         )
         writemsg_level(
-            "".join("!!! %s\n" % l for l in textwrap.wrap(msg, 70)),
+            "".join(f"!!! {l}\n" for l in textwrap.wrap(msg, 70)),
             level=logging.ERROR,
             noiselevel=-1,
         )
@@ -1188,10 +1162,12 @@ def profile_check(trees, myaction):
     return os.EX_OK
 
 
-def emerge_main(args=None):
+def emerge_main(args: Optional[list[str]] = None):
     """
-    @param args: command arguments (default: sys.argv[1:])
-    @type args: list
+    Entry point of emerge
+
+    Processes command line arguments (default: sys.argv[1:]) and decides
+    what the current run of emerge should by creating `emerge_config`
     """
     if args is None:
         args = sys.argv[1:]
@@ -1202,7 +1178,7 @@ def emerge_main(args=None):
     try:
         locale.setlocale(locale.LC_ALL, "")
     except locale.Error as e:
-        writemsg_level("setlocale: %s\n" % e, level=logging.WARN)
+        writemsg_level(f"setlocale: {e}\n", level=logging.WARN)
 
     # Disable color until we're sure that it should be enabled (after
     # EMERGE_DEFAULT_OPTS has been parsed).
@@ -1215,6 +1191,10 @@ def emerge_main(args=None):
     myaction, myopts, myfiles = parse_opts(args, silent=True)
     if "--debug" in myopts:
         os.environ["PORTAGE_DEBUG"] = "1"
+        portage.util.initialize_logger(logging.DEBUG)
+    else:
+        portage.util.initialize_logger()
+
     if "--config-root" in myopts:
         os.environ["PORTAGE_CONFIGROOT"] = myopts["--config-root"]
     if "--sysroot" in myopts:
@@ -1233,7 +1213,7 @@ def emerge_main(args=None):
         emerge_help()
         return os.EX_OK
     if myaction == "moo":
-        print(COWSAY_MOO % platform.system())
+        print(COWSAY_MOO.format(platform.system()))
         return os.EX_OK
     if myaction == "sync":
         # need to set this to True now in order for the repository config
@@ -1267,12 +1247,10 @@ def emerge_main(args=None):
             1: dev_null.fileno(),
             2: dev_null.fileno(),
         }
-        if (
-            portage.process.spawn_bash(
-                "[[ $(< <(echo foo) ) == foo ]]", fd_pipes=fd_pipes
-            )
-            != 0
-        ):
+        exit_code = portage.process.spawn_bash(
+            "[[ $(< <(echo foo) ) == foo ]]", fd_pipes=fd_pipes
+        )
+        if exit_code != 0:
             writemsg_level(
                 "Failed to validate a sane '/dev'.\n"
                 "bash process substitution doesn't work; this may be an "
@@ -1310,7 +1288,7 @@ def emerge_main(args=None):
     try:
         locale.setlocale(locale.LC_ALL, "")
     except locale.Error as e:
-        writemsg_level("setlocale: %s\n" % e, level=logging.WARN)
+        writemsg_level(f"setlocale: {e}\n", level=logging.WARN)
 
     tmpcmdline = []
     if "--ignore-default-opts" not in myopts:
