@@ -1,12 +1,10 @@
-# Copyright 2005-2022 Gentoo Authors
+# Copyright 2005-2023 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 import logging
 import re
 import subprocess
 
-from typing import Tuple
-
 import portage
 from portage import os
 from portage.util import writemsg_level, shlex_split
@@ -41,7 +39,7 @@ class GitSync(NewBase):
         """Tests whether the repo actually exists"""
         return os.path.exists(os.path.join(self.repo.location, ".git"))
 
-    def new(self, **kwargs) -> Tuple[int, bool]:
+    def new(self, **kwargs) -> tuple[int, bool]:
         """Do the initial clone of the repository"""
         if kwargs:
             self._kwargs(kwargs)
@@ -133,7 +131,7 @@ class GitSync(NewBase):
 
         return ":".join(directories)
 
-    def update(self) -> Tuple[int, bool]:
+    def update(self) -> tuple[int, bool]:
         """Update existing git repository, and ignore the syncuri. We are
         going to trust the user and assume that the user is in the branch
         that he/she wants updated. We'll let the user manage branches with
@@ -418,7 +416,16 @@ class GitSync(NewBase):
             )
             return False
 
-        openpgp_env = self._get_openpgp_env(self.repo.sync_openpgp_key_path)
+        opts = self.options.get("emerge_config").opts
+        debug = "--debug" in opts
+        quiet = self.settings.get("PORTAGE_QUIET") == "1"
+
+        openpgp_env = self._get_openpgp_env(self.repo.sync_openpgp_key_path, debug)
+
+        if debug:
+            old_level = logging.getLogger().getEffectiveLevel()
+            logging.getLogger().setLevel(logging.DEBUG)
+            logging.getLogger("gemato").setLevel(logging.DEBUG)
 
         try:
             out = EOutput()
@@ -453,7 +460,8 @@ class GitSync(NewBase):
                 return False
 
             if status == "G":  # good signature is good
-                out.einfo("Trusted signature found on top commit")
+                if not quiet:
+                    out.einfo("Trusted signature found on top commit")
                 return True
             if status == "U":  # untrusted
                 out.ewarn("Top commit signature is valid but not trusted")
@@ -473,12 +481,22 @@ class GitSync(NewBase):
             else:
                 expl = "unknown issue"
             out.eerror(f"No valid signature found: {expl}")
+
+            if debug:
+                writemsg_level(
+                    f"!!! Got following output from gpg: {status}\n",
+                    level=logging.DEBUG,
+                    noiselevel=-1,
+                )
+
             return False
         finally:
             if openpgp_env is not None:
                 openpgp_env.close()
+            if debug:
+                logging.getLogger().setLevel(old_level)
 
-    def retrieve_head(self, **kwargs) -> Tuple[int, bool]:
+    def retrieve_head(self, **kwargs) -> tuple[int, bool]:
         """Get information about the head commit"""
         if kwargs:
             self._kwargs(kwargs)
