@@ -16,8 +16,11 @@ __all__ = [
 ]
 
 import re
+import typing
 import warnings
 from functools import lru_cache
+from typing import Any, Optional, Union
+from collections.abc import Sequence
 
 
 import portage
@@ -28,7 +31,7 @@ portage.proxy.lazyimport.lazyimport(
     "portage.util:cmp_sort_key",
 )
 from portage import _unicode_decode
-from portage.eapi import _get_eapi_attrs
+from portage.eapi import _eapi_attrs, _get_eapi_attrs
 from portage.exception import InvalidData
 from portage.localization import _
 
@@ -76,7 +79,7 @@ endversion_keys = ["pre", "p", "alpha", "beta", "rc"]
 _slot_re_cache = {}
 
 
-def _get_slot_re(eapi_attrs):
+def _get_slot_re(eapi_attrs: _eapi_attrs) -> typing.re.Pattern:
     cache_key = eapi_attrs.slot_operator
     slot_re = _slot_re_cache.get(cache_key)
     if slot_re is not None:
@@ -96,7 +99,7 @@ def _get_slot_re(eapi_attrs):
 _pv_re = None
 
 
-def _get_pv_re(eapi_attrs):
+def _get_pv_re(eapi_attrs: _eapi_attrs) -> typing.re.Pattern:
     global _pv_re
     if _pv_re is not None:
         return _pv_re
@@ -106,7 +109,7 @@ def _get_pv_re(eapi_attrs):
     return _pv_re
 
 
-def ververify(myver, silent=1):
+def ververify(myver: str, silent: int = 1) -> bool:
     if ver_regexp.match(myver):
         return True
     if not silent:
@@ -115,7 +118,7 @@ def ververify(myver, silent=1):
 
 
 @lru_cache(1024)
-def vercmp(ver1, ver2, silent=1):
+def vercmp(ver1: str, ver2: str, silent: int = 1) -> Optional[int]:
     """
     Compare two versions
     Example usage:
@@ -262,7 +265,7 @@ def vercmp(ver1, ver2, silent=1):
     return rval
 
 
-def pkgcmp(pkg1, pkg2):
+def pkgcmp(pkg1: tuple[str, str, str], pkg2: tuple[str, str, str]) -> Optional[int]:
     """
     Compare 2 package versions created in pkgsplit format.
 
@@ -289,7 +292,7 @@ def pkgcmp(pkg1, pkg2):
     return vercmp("-".join(pkg1[1:]), "-".join(pkg2[1:]))
 
 
-def _pkgsplit(mypkg, eapi=None):
+def _pkgsplit(mypkg: str, eapi: Any = None) -> Optional[tuple[str, str, str]]:
     """
     @param mypkg: pv
     @return:
@@ -312,12 +315,16 @@ def _pkgsplit(mypkg, eapi=None):
     return (m.group("pn"), m.group("ver"), rev)
 
 
-_cat_re = re.compile("^%s$" % _cat, re.UNICODE)
+_cat_re = re.compile(f"^{_cat}$", re.UNICODE)
 _missing_cat = "null"
 
 
 @lru_cache(10240)
-def catpkgsplit(mydata, silent=1, eapi=None):
+def catpkgsplit(
+    mydata: Union[str, "_pkg_str"],
+    silent: int = 1,
+    eapi: Any = None,
+) -> Optional[tuple[str, ...]]:
     """
     Takes a Category/Package-Version-Rev and returns a list of each.
 
@@ -368,33 +375,33 @@ class _pkg_str(str):
 
     def __new__(
         cls,
-        cpv,
-        metadata=None,
-        settings=None,
-        eapi=None,
-        repo=None,
-        slot=None,
-        build_time=None,
-        build_id=None,
-        file_size=None,
-        mtime=None,
-        db=None,
+        cpv: str,
+        metadata: Optional[dict[str, Any]] = None,
+        settings: Any = None,
+        eapi: Any = None,
+        repo: Optional[str] = None,
+        slot: Optional[str] = None,
+        build_time: Optional[int] = None,
+        build_id: Optional[str] = None,
+        file_size: Optional[int] = None,
+        mtime: Optional[int] = None,
+        db: Any = None,
     ):
         return str.__new__(cls, cpv)
 
     def __init__(
         self,
-        cpv,
-        metadata=None,
-        settings=None,
-        eapi=None,
-        repo=None,
-        slot=None,
-        build_time=None,
-        build_id=None,
-        file_size=None,
-        mtime=None,
-        db=None,
+        cpv: str,
+        metadata: Optional[dict[str, Any]] = None,
+        settings: Any = None,
+        eapi: Any = None,
+        repo: Optional[str] = None,
+        slot: Optional[str] = None,
+        build_time: Optional[int] = None,
+        build_id: Optional[str] = None,
+        file_size: Optional[int] = None,
+        mtime: Optional[int] = None,
+        db: Any = None,
     ):
         if not isinstance(cpv, str):
             # Avoid TypeError from str.__init__ with PyPy.
@@ -456,13 +463,13 @@ class _pkg_str(str):
                 repo = _unknown_repo
             self.__dict__["repo"] = repo
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Any) -> None:
         raise AttributeError(
             "_pkg_str instances are immutable", self.__class__, name, value
         )
 
     @staticmethod
-    def _long(var, default):
+    def _long(var: Any, default: int) -> int:
         if var is not None:
             try:
                 var = int(var)
@@ -474,7 +481,7 @@ class _pkg_str(str):
         return var
 
     @property
-    def stable(self):
+    def stable(self) -> bool:
         try:
             return self._stable
         except AttributeError:
@@ -492,7 +499,9 @@ class _pkg_str(str):
             return stable
 
 
-def pkgsplit(mypkg, silent=1, eapi=None):
+def pkgsplit(
+    mypkg: str, silent: int = 1, eapi: Any = None
+) -> Optional[tuple[str, str, str]]:
     """
     @param mypkg: either a pv or cpv
     @return:
@@ -509,7 +518,7 @@ def pkgsplit(mypkg, silent=1, eapi=None):
     return (cat + "/" + pn, ver, rev)
 
 
-def cpv_getkey(mycpv, eapi=None):
+def cpv_getkey(mycpv: Union[_pkg_str, str], eapi: Any = None) -> Optional[str]:
     """Calls catpkgsplit on a cpv and returns only the cp."""
     try:
         return mycpv.cp
@@ -520,8 +529,7 @@ def cpv_getkey(mycpv, eapi=None):
         return mysplit[0] + "/" + mysplit[1]
 
     warnings.warn(
-        "portage.versions.cpv_getkey() "
-        + "called with invalid cpv: '{}'".format(mycpv),
+        "portage.versions.cpv_getkey() " + f"called with invalid cpv: '{mycpv}'",
         DeprecationWarning,
         stacklevel=2,
     )
@@ -536,7 +544,7 @@ def cpv_getkey(mycpv, eapi=None):
     return mysplit[0]
 
 
-def cpv_getversion(mycpv, eapi=None):
+def cpv_getversion(mycpv: Union[str, _pkg_str], eapi: Any = None) -> Optional[str]:
     """Returns the v (including revision) from an cpv."""
     try:
         return mycpv.version
@@ -548,7 +556,7 @@ def cpv_getversion(mycpv, eapi=None):
     return mycpv[len(cp + "-") :]
 
 
-def cpv_sort_key(eapi=None):
+def cpv_sort_key(eapi: Any = None) -> Any:
     """
     Create an object for sorting cpvs, to be used as the 'key' parameter
     in places like list.sort() or sorted(). This calls catpkgsplit() once for
@@ -563,8 +571,7 @@ def cpv_sort_key(eapi=None):
 
     split_cache = {}
 
-    def cmp_cpv(cpv1, cpv2):
-
+    def cmp_cpv(cpv1: Any, cpv2: Any) -> int:
         split1 = split_cache.get(cpv1, False)
         if split1 is False:
             split1 = None
@@ -597,11 +604,11 @@ def cpv_sort_key(eapi=None):
     return cmp_sort_key(cmp_cpv)
 
 
-def catsplit(mydep):
+def catsplit(mydep: str) -> list[str]:
     return mydep.split("/", 1)
 
 
-def best(mymatches, eapi=None):
+def best(mymatches: Sequence[Any], eapi: Any = None) -> Any:
     """Accepts None arguments; assumes matches are valid."""
     if not mymatches:
         return ""
